@@ -38,6 +38,7 @@ class Connection(object):
 
         attributes = user_data.get("attributes")
         if attributes is None:
+            logger.warning("LDAP user attributes empty")
             return None
 
         User = get_user_model()
@@ -65,9 +66,14 @@ class Connection(object):
             defaults=user_fields,
             **user_lookup
         )
+        # If the user was created, set them an unusable password.
+        if created:
+            user.set_unusable_password()
+            user.save()
         # Update relations
         import_func(settings.LDAP_AUTH_SYNC_USER_RELATIONS)(user, attributes)
         # All done!
+        logger.info("LDAP user lookup succeeded")
         return user
 
     def iter_users(self):
@@ -107,6 +113,7 @@ class Connection(object):
             size_limit=1,
         ):
             return self._get_or_create_user(self._connection.response[0])
+        logger.warning("LDAP user lookup failed")
         return None
 
 
@@ -119,6 +126,7 @@ def connection(**kwargs):
     in settings.LDAP_AUTH_USER_LOOKUP_FIELDS, plus a `password` argument.
     """
     # Format the DN for the username.
+    format_username = import_func(settings.LDAP_AUTH_FORMAT_USERNAME)
     kwargs = {
         key: value
         for key, value
@@ -129,28 +137,56 @@ def connection(**kwargs):
     password = None
     if kwargs:
         password = kwargs.pop("password")
-        username = import_func(settings.LDAP_AUTH_FORMAT_USERNAME)(kwargs)
-    # Make the connection.
+        username = format_username(kwargs)
+    # Configure the connection.
     if settings.LDAP_AUTH_USE_TLS:
         auto_bind = ldap3.AUTO_BIND_TLS_BEFORE_BIND
     else:
         auto_bind = ldap3.AUTO_BIND_NO_TLS
+    # Connect.
     try:
-        with ldap3.Connection(ldap3.Server(
-            settings.LDAP_AUTH_URL,
-            allowed_referral_hosts=[("*", True)]),
+        c = ldap3.Connection(
+            ldap3.Server(
+                settings.LDAP_AUTH_URL,
+                allowed_referral_hosts=[("*", True)],
+                get_info=ldap3.NONE,
+            ),
             user=username,
             password=password,
             auto_bind=auto_bind,
             raise_exceptions=True,
-        ) as c:
-            yield Connection(c)
+        )
     except LDAPException as ex:
-        logger.info("LDAP connect failed: {ex}".format(ex=ex))
+        logger.warning("LDAP connect failed: {ex}".format(ex=ex))
         yield None
+        return
+    # If the settings specify an alternative username and password for querying, rebind as that.
+    if (
+        (settings.LDAP_AUTH_CONNECTION_USERNAME or settings.LDAP_AUTH_CONNECTION_PASSWORD) and
+        (
+            settings.LDAP_AUTH_CONNECTION_USERNAME != username or
+            settings.LDAP_AUTH_CONNECTION_PASSWORD != password
+        )
+    ):
+        User = get_user_model()
+        try:
+            c.rebind(
+                user=format_username({User.USERNAME_FIELD: settings.LDAP_AUTH_CONNECTION_USERNAME}),
+                password=settings.LDAP_AUTH_CONNECTION_PASSWORD,
+            )
+        except LDAPException as ex:
+            logger.warning("LDAP rebind failed: {ex}".format(ex=ex))
+            yield None
+            return
+    # Return the connection.
+    logger.info("LDAP connect succeeded")
+    try:
+        yield Connection(c)
+    finally:
+        c.unbind()
 
 
-def authenticate(**kwargs):
+def authenticate(*args, **kwargs):
     """
     Authenticates with the LDAP server, and returns
     the corresponding Django user instance.
